@@ -70,7 +70,8 @@ ngx_http_lua_ssl_cert_handler_inline(ngx_http_request_t *r,
                                        lscf->srv.ssl_cert_src.len,
                                        &lscf->srv.ssl_cert_src_ref,
                                        lscf->srv.ssl_cert_src_key,
-                                       "=ssl_certificate_by_lua");
+                                       (const char *)
+                                       lscf->srv.ssl_cert_chunkname);
     if (rc != NGX_OK) {
         return rc;
     }
@@ -115,6 +116,8 @@ ngx_http_lua_ssl_cert_by_lua(ngx_conf_t *cf, ngx_command_t *cmd,
 
 #else
 
+    size_t                       chunkname_len;
+    u_char                      *chunkname;
     u_char                      *cache_key = NULL;
     u_char                      *name;
     ngx_str_t                   *value;
@@ -163,8 +166,15 @@ ngx_http_lua_ssl_cert_by_lua(ngx_conf_t *cf, ngx_command_t *cmd,
             return NGX_CONF_ERROR;
         }
 
+        chunkname = ngx_http_lua_gen_chunk_name(cf, "ssl_certificate_by_lua",
+                          sizeof("ssl_certificate_by_lua") - 1, &chunkname_len);
+        if (chunkname == NULL) {
+            return NGX_CONF_ERROR;
+        }
+
         /* Don't eval nginx variables for inline lua code */
         lscf->srv.ssl_cert_src = value[1];
+        lscf->srv.ssl_cert_chunkname = chunkname;
     }
 
     lscf->srv.ssl_cert_src_key = cache_key;
@@ -1042,7 +1052,7 @@ ngx_http_lua_ffi_cert_pem_to_der(const u_char *pem, size_t pem_len, u_char *der,
 
 int
 ngx_http_lua_ffi_priv_key_pem_to_der(const u_char *pem, size_t pem_len,
-    u_char *der, char **err)
+    const u_char *passphrase, u_char *der, char **err)
 {
     int          len;
     BIO         *in;
@@ -1055,7 +1065,7 @@ ngx_http_lua_ffi_priv_key_pem_to_der(const u_char *pem, size_t pem_len,
         return NGX_ERROR;
     }
 
-    pkey = PEM_read_bio_PrivateKey(in, NULL, NULL, NULL);
+    pkey = PEM_read_bio_PrivateKey(in, NULL, NULL, (void *) passphrase);
     if (pkey == NULL) {
         BIO_free(in);
         *err = "PEM_read_bio_PrivateKey() failed";
@@ -1479,4 +1489,15 @@ failed:
 }
 
 
+ngx_ssl_conn_t *
+ngx_http_lua_ffi_get_req_ssl_pointer(ngx_http_request_t *r)
+{
+    if (r->connection == NULL || r->connection->ssl == NULL) {
+        return NULL;
+    }
+
+    return r->connection->ssl->connection;
+}
+
+
 #endif /* NGX_HTTP_SSL */
